// PlayerWidget.cpp //

#include "PlayerWidget.h"
#include "core_ex.h"
#include <sstream>
#include <string>

#include "UI.h"

using std::stringstream;
using std::string;

PlayerWidget :: PlayerWidget (vector< Player > *poPlayer)
  : Widget(c_kiYSize+3, c_kiXSize+4, c_kiYPos, c_kiXPos, true), 
   m_poVecPlayers(NULL), m_iCurrIndex(0)
{
    m_poVecPlayers = poPlayer;
    keypad(m_poWin, true);
}

void PlayerWidget :: Draw (void)
{
  DrawPlayer();
}

int PlayerWidget :: KeyFocus (void)
{
  if (m_poVecPlayers->size()<c_kiMaxPlayers)
  {
    UI::Instance()->Status("Enter player names.");
  }
  else
  {
    UI::Instance()->Status("Maximum number of players in game.");
  }

  while (true)
  {
    curs_set(1);
    int iCh = GetCh(c_kiLHeight+1+m_iCurrIndex
        , c_kiIDWidth+1+m_oStrEditBuf.size());
    switch (iCh)
    {
      case 9:
        UI::Instance()->Status("Leaving Player Widget.");
        m_oStrEditBuf = "";
        Render();
        return iCh;
        break;

      case KEY_BACKSPACE:
        if (m_oStrEditBuf.length() > 0)
        {
          // delete only one char 
          m_oStrEditBuf.erase(m_oStrEditBuf.length()-1);
          AddStr(c_kiLHeight+1+m_iCurrIndex, c_kiIDWidth+1, 
              m_oStrEditBuf);
          for(int iXPos = c_kiIDWidth+1 + m_oStrEditBuf.length();
              iXPos < c_kiIDWidth + c_kiNWidth;
              iXPos++)
          {
            AddStr(c_kiLHeight+1+m_iCurrIndex, iXPos, "_");
          }
        }
        else
        {
          if (0 < m_iCurrIndex)
          {
            m_poVecPlayers->pop_back();
            m_iCurrIndex--;
            m_oStrEditBuf = "";
            UI::Instance()->Status("Enter player names.");
            Render();
          }
          else
          {
            beep();
            UI::Instance()->Status("No more players to delete.");
          }
        }
        break;

      case 10:
        try
        {
          if (m_iCurrIndex < c_kiMaxPlayers)
          {
            Player oPlayer('A'+m_iCurrIndex, m_oStrEditBuf);
            m_poVecPlayers->push_back(oPlayer);
            m_iCurrIndex++;

            if (m_iCurrIndex < c_kiMaxPlayers) 
            {
              UI::Instance()->Status("Enter player names.");
            }
            else
            {
              UI::Instance()->Status("Maximum number of players reached.");
            }
          }
          else
          {
            beep();
            UI::Instance()->Status("Cannot add any more players.");
          }
          m_oStrEditBuf = "";
          Render();
        }
        catch (BadPNameEx& rErr)
        {
          beep();
          UI::Instance()->Status(rErr.what());
        }
        break;

      case 'a'... 'z':
      case 'A'... 'Z':
      /*case '0'... '9':*/
      case ' ':
        {
          if ((m_iCurrIndex < c_kiMaxPlayers) && (m_oStrEditBuf.size() < 
              c_kiNWidth-1))
          {
            m_oStrEditBuf += iCh;
            wattron(m_poWin,WA_BOLD);
            AddStr(c_kiLHeight+1+m_iCurrIndex, c_kiIDWidth+1, 
                m_oStrEditBuf);
            wattroff(m_poWin,WA_BOLD);
          }
          else
          {
            beep();
            UI::Instance()->Status("Invalid character for player name.");
          }
        }
        break;

      default:
        // do nothing
        beep();
        break;
    }
  }
  return 0;
}

void PlayerWidget :: DrawPlayer (void)
{
  AddStr(0,0," ID");
  VLine(c_kiIDWidth);
  AddStr(0,c_kiIDWidth+1," Name");
  VLine(c_kiIDWidth + c_kiNWidth + 1);
  AddStr(0,c_kiIDWidth + c_kiNWidth + 2," Pos");
  HLine(1);
  for (int i=0; i<c_kiMaxPlayers; i++)
  {
    string oStrID;
    string oStrName;
    string oStrPos;
    if (i<m_poVecPlayers->size())
    {
      oStrID += (*m_poVecPlayers)[i].GetPID();
      oStrName += (*m_poVecPlayers)[i].GetPName();
      stringstream oSrmPos;
      oSrmPos.width(2);
      oSrmPos << (*m_poVecPlayers)[i].GetCurPos() + 1;
      oStrPos += oSrmPos.str(); 
    }
    while (oStrID.size() < c_kiIDWidth-2)
    {
      oStrID += '.';
    }
    while (oStrName.size() < c_kiNWidth-1)
    {
      oStrName += '_';
    }
    while (oStrPos.size() < c_kiPWidth-1)
    {
      oStrPos += ' ';
    }
    AddStr(c_kiLHeight+i+1,1,oStrID);
    AddStr(c_kiLHeight+i+1,c_kiIDWidth+1,oStrName);
    AddStr(c_kiLHeight+i+1,c_kiNWidth+c_kiIDWidth+3,oStrPos);
  }
}

int PlayerWidget :: GetPlayerNum (void)
{
  return m_poVecPlayers->size();
}

void PlayerWidget::Clear (void)
{
  m_iCurrIndex = 0;
  m_poVecPlayers->clear();
  Render();
}

// PlayerWidget.cpp //
