// UI.cpp //

#include <curses.h>
#include <unistd.h>
#include <sstream>
#include <string>
#include <fstream>
#include <iostream>

#include "UI.h"
#include "Game.h"
#include "DialogWidget.h"
#include "EntryWidget.h"
#include "GraphDieWidget.h"

using std::stringstream;

UI *UI::c_poUI=NULL;

/// Initializer instance method
UI *UI::Instance (Game *poGame)
{
  if (c_poUI == NULL)
  {
    c_poUI = new UI(poGame);
    return c_poUI;
  }
  else
  {
    throw UIInstantiated();
  }
}
    
/// access instance
UI *UI::Instance (void)
{
  if (c_poUI != NULL)
  {
    return c_poUI;
  }
  else
  {
    throw UINotInstantiated();
  }
}

void UI::Destroy (void)
{
  if (c_poUI != NULL)
  {
    delete c_poUI;
    c_poUI = NULL;
  }
  else
  {
    throw UINotInstantiated();
  }
}

UI :: UI (Game *poGame)
{
  try
  {
    initscr();
    cbreak();
    noecho();
    start_color();

    // define a new menu
    vector< string > oVecMenuEntries1;
    oVecMenuEntries1.push_back("Players");
    oVecMenuEntries1.push_back("Start");
    oVecMenuEntries1.push_back("New");
    oVecMenuEntries1.push_back("Exit");
    m_poMenuWid1 = new MenuWidget(0,0,"Game",oVecMenuEntries1);
    
    // define a new menu
    vector< string > oVecMenuEntries2;
    oVecMenuEntries2.push_back("New");
    oVecMenuEntries2.push_back("Load");
    oVecMenuEntries2.push_back("Edit");
    oVecMenuEntries2.push_back("Save");
    oVecMenuEntries2.push_back("Delete");
    m_poMenuWid2 =new MenuWidget(0,m_poMenuWid1->GetWidth()+1,"Board",
        oVecMenuEntries2);

    // define a new menu
    vector< string > oVecMenuEntries3;
    oVecMenuEntries3.push_back("About");
    int iXMenu3Pos = m_poMenuWid1->GetWidth() + m_poMenuWid2->GetWidth() + 2;
    m_poMenuWid3 =new MenuWidget(0,iXMenu3Pos,"Help",
        oVecMenuEntries3);

    m_poMenuWid1->SetPrevMenu(m_poMenuWid3);
    m_poMenuWid1->SetNextMenu(m_poMenuWid2);
    m_poMenuWid2->SetPrevMenu(m_poMenuWid1);
    m_poMenuWid2->SetNextMenu(m_poMenuWid3);
    m_poMenuWid3->SetPrevMenu(m_poMenuWid2);
    m_poMenuWid3->SetNextMenu(m_poMenuWid1);

    m_poGame = poGame;
    
    m_poBoardWid = new BoardWidget(poGame);
    
    m_poStatusWid = new StatusWidget();
    
    m_poDieWid = new GraphDieWidget(poGame->GetDie());
    
    m_poPlayerWid = new PlayerWidget(poGame->GetPlayers());
    
  }
  catch (runtime_error& rErr)
  {
    echo();
    nocbreak();
    endwin();
    throw;
  }
}

UI :: ~UI (void)
{
  echo();
  nocbreak();
  endwin();
}

void UI :: Run (void)
{
  m_poMenuWid1->Render();
  m_poMenuWid2->Render();
  m_poMenuWid3->Render();
  m_poBoardWid->Render();
  m_poPlayerWid->Render();
  m_poDieWid->Render();
  Refresh();

  Widget *poFocusWid = m_poBoardWid;
  Status("<TAB> - Menu");
  while (true)
  {
    //m_poBoardWid->Render();
    int iCh = poFocusWid->KeyFocus();
    switch(iCh)
    {
      case 9: 
        if (m_poGame->IsStarted())
        {
          string ostrHelp;

          ostrHelp = (string)"\n" +
            " Do you want to abort\n" + 
            " the current game (y/n)?";
          DialogWidget oDialogWid = DialogWidget(6,27,9,25, ostrHelp);
          oDialogWid.Render();
          Refresh();

          char iCh = oDialogWid.KeyFocus();
          switch (iCh)
          {
            case 'y':
            case 'Y':
              {
                m_poGame->Stop();
                Status("Game aborted. <TAB> - Menu");
                poFocusWid = m_poBoardWid;
              }
              break;

            case 'n':
            case 'N':
            default:
              {
                PromptNextMove();
              }
              break;
          }
          oDialogWid.Hide();
          Refresh();
        }
        else
        {
          MenuWidget *poMenuWid = m_poMenuWid1;
          int iMenuCh;
          bool bExitMenu = false;
          do 
          {
            iMenuCh = poMenuWid->KeyFocus();

            switch (iMenuCh)
            {
              case 27:
                bExitMenu = true;
                Status("<TAB> - Menu");
                break;

              case KEY_RIGHT:
                poMenuWid = poMenuWid->GetNextMenu();
                break;
            
              case KEY_LEFT:
                poMenuWid = poMenuWid->GetPrevMenu();
                break;
            
              case 10:
                if (poMenuWid == m_poMenuWid1 )
                {
                  if (poMenuWid->Selection() == "Start")
                  {
                    try
                    {
                      m_poGame->Start();
                      m_poBoardWid->Render();
                      m_poPlayerWid->Render();
                      Refresh();
                      PromptNextMove();
                      poFocusWid = m_poDieWid;
                    }
                    catch (NotEnoughPlayersEx eEx)
                    {
                      Status(eEx.what());
                      beep();
                      poFocusWid = m_poPlayerWid;
                    }
                    catch (GameAlreadyOnEx& rErr)
                    {
                      Status(rErr.what());
                      beep();
                      sleep(1);
                      poFocusWid = m_poDieWid;
                    }   
                  }
                  else if (poMenuWid->Selection() == "New")
                  {
                    m_poPlayerWid->Clear();
                    m_poBoardWid->Render();
                    Refresh();
                    m_poGame->Stop();
                    poFocusWid = m_poPlayerWid;
                  }
                  else if ((poMenuWid->Selection() == "Players") && 
                      (m_poGame->IsStarted()))
                  {
                    Status("Game already on, cannot add players now.");
                    beep();
                    sleep(1);
                  }
                  else if ((poMenuWid->Selection() == "Players") && 
                    (!m_poGame->IsStarted()))
                  {
                    poFocusWid = m_poPlayerWid;
                  }
                  else if (poMenuWid->Selection() == "Exit")
                  {
                    return;
                  }
                  else
                  {
                    iCh = 0;
                  }
                }
                else if (poMenuWid == m_poMenuWid2 )
                {
                  m_poGame->Stop();
                  m_poPlayerWid->Render();
                  m_poBoardWid->Render();
                  if (poMenuWid->Selection() == "New")
                  {
                    try
                    {
                      m_poGame->LoadEmpty();
                      Status("New board");
                      poFocusWid = m_poBoardWid;
                      m_poBoardWid->Render();
                    }
                    catch(runtime_error& rErr)
                    {
                      Status(rErr.what());
                    }
                  }
                  else if (poMenuWid->Selection() == "Load")
                  {
                    // define a sub menu
                    MenuWidget *poSubMenuWid;
                    vector< string > oVecSubMenuEntries;
                    oVecSubMenuEntries = BoardFactory::Instance()->List(); 
                    int iSMXPos = m_poMenuWid1->GetWidth() + 
                      m_poMenuWid2->GetWidth() + 1;
                    poSubMenuWid = new MenuWidget(1, iSMXPos + 2, " ", 
                        oVecSubMenuEntries);
                    poSubMenuWid->Render();
                    Refresh();
                    poSubMenuWid->KeyFocus();
                    try
                    {
                      m_poGame->Load(poSubMenuWid->Selection());
                      Status("Loaded " +  poSubMenuWid->Selection());
                      poFocusWid = m_poBoardWid;
                      m_poBoardWid->Render();
                    }
                    catch(runtime_error& rErr)
                    {
                      Status(rErr.what());
                    }
                  }
                  else if (poMenuWid->Selection() == "Edit")
                  {
                    try
                    {
                      Status("Edit");
                      m_poBoardWid->Edit();
                      poFocusWid = m_poBoardWid;
                      m_poBoardWid->Render();
                    }
                    catch (runtime_error& rErr)
                    {
                      Status ((string) "Edit board failed: " + rErr.what());
                    }
                  }
                  else if (poMenuWid->Selection() == "Save")
                  {
                    try
                    {
                      EntryWidget oEntWid(10,25,30, "Enter filename");
                      oEntWid.KeyFocus();
                      BoardFactory::Instance()->SaveBoard(m_poGame->GetBoard(), 
                          oEntWid.GetValue());
                      Status("Saved to " + oEntWid.GetValue());
                      BoardFactory::Instance()->ReadList();
                    }
                    catch (runtime_error& rErr)
                    {
                      Status((string) "Board save failed: " + rErr.what());
                    }
                  }
                  else if (poMenuWid->Selection() == "Delete")
                  {
                    // define a sub menu
                    MenuWidget *poSubMenuWid;
                    vector< string > oVecSubMenuEntries;
                    oVecSubMenuEntries = BoardFactory::Instance()->List(); 
                    int iSMXPos = m_poMenuWid1->GetWidth() + 
                      m_poMenuWid2->GetWidth() + 1;
                    poSubMenuWid = new MenuWidget(5, iSMXPos + 2, " ", 
                        oVecSubMenuEntries);
                    poSubMenuWid->Render();
                    Refresh();
                    poSubMenuWid->KeyFocus();
                    try
                    {
                      BoardFactory::Instance()->Delete(
                          poSubMenuWid->Selection());
                      Status("Deleted Board : " +  poSubMenuWid->Selection());
                      BoardFactory::Instance()->ReadList();
                    }
                    catch(runtime_error& rErr)
                    {
                      Status(rErr.what());
                    }
                  }
                  else
                  {
                    iCh = 0;
                  }
                }
                else if (poMenuWid == m_poMenuWid3 )
                {
                  string ostrHelp;
                  if (poMenuWid->Selection() == "About")
                  {
                    stringstream oSrm("");
                    oSrm << "Snakes & Ladders v" << PACKAGE_VERSION;
                    ostrHelp = oSrm.str();
                    ostrHelp += 
                    (string) "\n" 
                    + "------------------------------------" 
                    + "\n" + "\n" 
                    + " Aruna M. Naik           " + "\n" 
                    + "                         " + "\n" 
                    + " v1.0 Authors:           " + "\n" 
                    + " Aruna M. Naik           " + "\n" 
                    + " Vanessa Noronha         " + "\n" 
                    + " Rakhi Shah              " + "\n" 
                    + " Veena Bhosle            " + "\n";
                    DialogWidget oDialogWid = DialogWidget(12,40,6,20, ostrHelp);
                    oDialogWid.Render();
                    Refresh();
                    oDialogWid.KeyFocus();
                    Status("<TAB> - Menu");
                  }
                }
                else
                {
                  // can not happen
                }
                
                bExitMenu = true;
                m_poBoardWid->Top();
                Refresh();
                break;

              default:
                beep();
                break;
            }
            m_poBoardWid->Top();
            Refresh();
          }
          while (!bExitMenu);
        } 
      break;

      case 10: 
      {
        try 
        {
          if (!m_poGame->IsStarted())
          {
            break;
          }
          int iMove = m_poGame->GetDie()->GetFaceVal();
          ShowLastMove();
          try
          {
            m_poGame->Play(iMove);
          }
          catch (GameStartEx& rErr)
          {
            beep();
            UI::Instance()->Status(rErr.what());
            sleep(1);
          }
          catch (OverThrowEx& rErr)
          {
            UI::Instance()->Status(rErr.what());
            sleep(1);
          }
          m_poBoardWid->Render();
          m_poPlayerWid->Render();
          Refresh();
          napms(1000);
          PromptNextMove();
        }
        catch (GameNotOnEx& rErr)
        {
          beep();
          //UI::Instance()->Status(rErr.what());
          sleep(1);
        }
        catch (GameOverEx& rErr)
        {
          poFocusWid = m_poBoardWid;
          m_poBoardWid->Render();
          m_poPlayerWid->Render();
          Refresh();
          m_poGame->Stop();
          beep();
          usleep(500000);
          beep();
          usleep(500000);
          beep();
          UI::Instance()->Status(rErr.what());
          sleep(5);
        }
      }
      break; 
      
      default:
        break;
    }
  } 
}

void UI::Refresh (void)
{
  update_panels();
  doupdate();
}

void UI::Status (const string& rkoStrMsg)
{
  m_poStatusWid->PutMsg(rkoStrMsg);
  m_poStatusWid->Render();

}

void UI::PromptNextMove (void)
{
  stringstream oSrmPID;
  oSrmPID.width(1);
  oSrmPID << m_poGame->GetCurrPlayer().GetPID();
  string ostr;
  ostr += oSrmPID.str();
  Status(ostr + " : " + m_poGame->GetCurrPlayer().GetPName()
      + "'s move. <ENTER> to roll die");
}

void UI::ShowLastMove (void)
{
  int iMove = m_poGame->GetDie()->GetFaceVal();
  stringstream oSrmPID;
  oSrmPID.width(1);
  oSrmPID << m_poGame->GetCurrPlayer().GetPID()
    << " : " << m_poGame->GetCurrPlayer().GetPName()
    << " threw a " << iMove;
  string ostr;
  ostr += oSrmPID.str();
  Status(ostr);
}
// UI.cpp //
